/* GNOME DB library
 * Copyright (C) 1999-2002 The GNOME Foundation.
 *
 * AUTHORS:
 *      Daniel Espinosa <esodan@yahoo.com.mx>
 *
 * This Library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This Library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this Library; see the file COPYING.LIB.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#include "gnome-db-field-editor.h"


#define PARENT_TYPE GTK_TYPE_VBOX


struct _GnomeDbFieldEditorPrivate {
	GdaDataModel *model;
	gint row;
	gint col;
	GtkWidget *label;
	GtkWidget *widget;
	gboolean show_label;
	GnomeDbFieldEditorLabelPosition label_position;
};

static void gnome_db_field_editor_class_init   (GnomeDbFieldEditorClass *klass);
static void gnome_db_field_editor_init         (GnomeDbFieldEditor *field_editor, GnomeDbFieldEditorClass *klass);
static void gnome_db_field_editor_set_property (GObject *object,
					guint paramid,
					const GValue *value,
					GParamSpec *pspec);
static void gnome_db_field_editor_get_property (GObject *object,
					guint param_id,
					GValue *value,
					GParamSpec *pspec);
static void gnome_db_field_editor_finalize     (GObject *object);

static void update_value(GnomeDbFieldEditor *field_editor);
static void set_value(GnomeDbFieldEditor *field_editor, GdaValue *value);
static void download_file(GtkButton *button, gpointer data);
static void upload_file(GtkButton *button, gpointer data);
static void create_widgets(GnomeDbFieldEditor *field_editor);

enum {
	MODEL_CHANGED,
	LAST_SIGNAL
};

static gint field_editor_signals[LAST_SIGNAL];
static GObjectClass *parent_class = NULL;

/*
 * Callbacks
 */


/*
 * Private functions
 */

static void
set_value(GnomeDbFieldEditor *field_editor, GdaValue *value)
{
	if(GTK_IS_ENTRY(field_editor->priv->widget))
	{
		gtk_entry_set_text(field_editor->priv->widget, 
							gda_value_stringify(value));
	}
	
	if(GTK_IS_CHECK_BUTTON(field_editor->priv->widget))
	{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(field_editor->priv->widget),
											gda_value_get_boolean(value));
	}
	
	if(GTK_IS_VBOX(field_editor->priv->widget))
	{
		
	}
}

static void
update_value(GnomeDbFieldEditor *field_editor)
{
	if(GTK_IS_ENTRY(field_editor->priv->widget))
	{
/* 		g_message("Es un Entry -- Val = %s", 
						gda_value_stringify(
								gda_data_model_get_value_at(
										field_editor->priv->model,
										field_editor->priv->col,
										field_editor->priv->row))); */
		
		gchar *txt;

		txt = gnome_db_field_editor_get_value_text(field_editor);
		
		
		gtk_entry_set_text(GTK_ENTRY(field_editor->priv->widget),	txt);
		g_message("Es un Entry -- Val = %s", 
						gnome_db_field_editor_get_value_text(field_editor));
	}
	
	if(GTK_IS_TOGGLE_BUTTON(field_editor->priv->widget))
	{
		g_message("Es un Boolean -- Val = %s", 
					gda_value_stringify(
							gda_data_model_get_value_at(field_editor->priv->model,
														field_editor->priv->col,
														field_editor->priv->row)));
	
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(field_editor->priv->widget),
											gda_value_get_boolean(
												gda_data_model_get_value_at(
													field_editor->priv->model,
													field_editor->priv->col,
													field_editor->priv->row)));
	}
	
	if(GTK_IS_VBOX(field_editor->priv->widget))
	{
		
	}
	
}

static void 
create_widgets(GnomeDbFieldEditor *field_editor) 
{
	GdaValueType type;
	GtkWidget *box;
	
	g_return_if_fail (GNOME_DB_IS_FIELD_EDITOR(field_editor));
			
	if(GTK_IS_WIDGET(field_editor->priv->widget))
		gtk_widget_destroy(field_editor->priv->widget);
	
	type = gnome_db_field_editor_get_field_editor_type(field_editor);
	
	switch(type) {
		case GDA_VALUE_TYPE_GEOMETRIC_POINT:
			/* Three GtkFieldEditor widgets*/
			break;	
		case GDA_VALUE_TYPE_NULL:
		case GDA_VALUE_TYPE_BIGINT:
		case GDA_VALUE_TYPE_BIGUINT:
		case GDA_VALUE_TYPE_DOUBLE:
		case GDA_VALUE_TYPE_INTEGER:
		case GDA_VALUE_TYPE_MONEY:
		case GDA_VALUE_TYPE_NUMERIC:
		case GDA_VALUE_TYPE_SMALLINT:
		case GDA_VALUE_TYPE_SMALLUINT:
		case GDA_VALUE_TYPE_STRING:
		case GDA_VALUE_TYPE_TIME:
		case GDA_VALUE_TYPE_TIMESTAMP:
		case GDA_VALUE_TYPE_TINYINT:
		case GDA_VALUE_TYPE_TINYUINT:
		case GDA_VALUE_TYPE_DATE: 
			field_editor->priv->widget = gtk_entry_new();
			break;
		
		case GDA_VALUE_TYPE_BINARY:
		case GDA_VALUE_TYPE_BLOB:
			/*Button widget for File Selection to Upload/Download files*/
			{
				GtkButton *button_download, *button_upload;
												
				button_download = gtk_button_new_from_stock("gtk-go-down");
				button_upload = gtk_button_new_from_stock("gtk-go-up");
				
				field_editor->priv->widget = gtk_vbox_new(TRUE, 20);
				
				gtk_box_pack_start_defaults(GTK_BOX(field_editor->priv->widget), button_download);
				gtk_box_pack_start_defaults(GTK_BOX(field_editor->priv->widget), button_upload);
				
				g_signal_connect(G_OBJECT(button_download), "clicked", download_file, field_editor);
				g_signal_connect(G_OBJECT(button_download), "clicked", upload_file, field_editor);
				
			}
			break;
		
		case GDA_VALUE_TYPE_BOOLEAN:
			
			/* Gtk widget */
			field_editor->priv->widget = gtk_toggle_button_new();
			break;
		
		
		case GDA_VALUE_TYPE_GOBJECT:
		case GDA_VALUE_TYPE_LIST:
		case GDA_VALUE_TYPE_SINGLE:
		case GDA_VALUE_TYPE_TYPE:
		case GDA_VALUE_TYPE_UNKNOWN:
			/* With out widget*/
			
			field_editor->priv->widget = gtk_label_new("Can't show Data");
	}
	
	if(!field_editor->priv->show_label)
	{
		if(GTK_IS_WIDGET(field_editor->priv->label))
				gtk_widget_destroy(field_editor->priv->label);
		
		field_editor->priv->label = gtk_label_new(NULL);
		GString *label = g_string_new(NULL);
		g_string_append(label, "<span style='italic' weight='bold'>_");
		g_string_append(label, gnome_db_field_editor_get_column_name(field_editor));
		g_string_append(label, "</span>");
		gtk_label_set_markup_with_mnemonic(field_editor->priv->label, label->str);
		gtk_label_set_mnemonic_widget(field_editor->priv->label, field_editor->priv->widget);
		g_string_free(label, TRUE);
	}
	
	
	switch (field_editor->priv->label_position)
	{
		case GNOME_DB_FIELD_EDITOR_LABEL_BEHIND:
			box = gtk_vbox_new(FALSE,0);
			gtk_box_pack_start(GTK_BOX(box), field_editor->priv->widget, FALSE, FALSE, 0);
			gtk_box_pack_start(GTK_BOX(box), field_editor->priv->label, FALSE, FALSE, 0);
			break;
		case GNOME_DB_FIELD_EDITOR_LABEL_LEFT:
			box = gtk_hbox_new(FALSE,0);
			gtk_box_pack_start(GTK_BOX(box), field_editor->priv->label, FALSE, FALSE, 0);
			gtk_box_pack_start(GTK_BOX(box), field_editor->priv->widget, FALSE, FALSE, 0);
			break;
		case GNOME_DB_FIELD_EDITOR_LABEL_RIGHT:
			box = gtk_hbox_new(FALSE,0);
			gtk_box_pack_start(GTK_BOX(box), field_editor->priv->widget, FALSE, FALSE, 0);
			gtk_box_pack_start(GTK_BOX(box), field_editor->priv->label, FALSE, FALSE, 0);
			break;
		case GNOME_DB_FIELD_EDITOR_LABEL_ON_TOP:
		default :
			box = gtk_vbox_new(FALSE,0);
			gtk_box_pack_start(GTK_BOX(box), field_editor->priv->label, FALSE, FALSE, 0);
			gtk_box_pack_start(GTK_BOX(box), field_editor->priv->widget, FALSE, FALSE, 0);
			break;
	}
	gtk_box_pack_start(GTK_BOX(field_editor), box, FALSE, FALSE, 0);
	
	gtk_widget_show_all(GTK_WIDGET(field_editor));
}

static void
download_file (GtkButton *button, gpointer data) 
{
/*
	GtkFileChooserDialog file_chooser;
	GnomeDbFieldEditor *field_editor = (GnomeDbFieldEditor*) data;
	
	g_return_if_fail(GNOMEDB_IS_FIELD_EDITOR(field_editor));
	
	file_chooser = gtk_file_chooser_dialog_new("Download a Binary Object...", 
												NULL, 
												GTK_FILE_CHOOSER_ACTION_SAVE,
												GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
												GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
	if(gtk_dialog_run(GTK_DIALOG(file_chooser)) == GTK_RESPONSE_ACCEPT)
	{
		gchar *filename;
		gchar *buffer;
		gint file_len;
		GdaBlob *blob;
		
		blob = gda_value_get_blob(gda_data_model_get_value_at (field_editor->priv->model,
																field_editor->priv->col,
																field_editor->priv->row));
		
		if(!gda_blob_open(blob, GDA_BLOB_MODE_READ))
		{
			buffer = g_try_malloc(
			filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_chooser));
			g_file_get_contents(filename, &buffer, 
			
			
			g_free(buffer);
		}
		g_free(blob);
		
	}
	*/
}

static void
upload_file (GtkButton *button, gpointer data) 
{
/*	GtkFileChooserDialog file_chooser;
	
	file_chooser = gtk_file_chooser_dialog_new("Upload a Binary Object...", 
												GTK_WIDGET(data), 
												GTK_FILE_CHOOSER_ACTION_SAVE,
												GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
												GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
	if(gtk_dialog_run(GTK_DIALOG(file_chooser)) == GTK_RESPONSE_ACCEPT)
	{
		gchar *filename;
		gchar *buffer;
		gint file_len;
		GdaBlob *blob, *new_blob;
		
		blob = gda_value_get_blob(gda_data_model_get_value_at (field_editor->priv->model,
																field_editor->priv->col,
																field_editor->priv->row));
		
		if(!gda_blob_open(blob, GDA_BLOB_MODE_READ))
		{
			new_blob = (GdaBlob*) g_newa(GdaBlob,1);
			filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_chooser));
			gda_file_load(filename, buffer, file_len);
		}
	}
	*/
}


/*
 * GnomeDbFieldEditor class implementation
 */

static void
gnome_db_field_editor_class_init (GnomeDbFieldEditorClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	parent_class = g_type_class_peek_parent (klass);

	object_class->set_property = gnome_db_field_editor_set_property;
	object_class->get_property = gnome_db_field_editor_get_property;
	object_class->finalize = gnome_db_field_editor_finalize;

	/* add class signals */
	
}

static void
gnome_db_field_editor_init (GnomeDbFieldEditor *field_editor, GnomeDbFieldEditorClass *klass)
{
	g_return_if_fail (GNOME_DB_IS_FIELD_EDITOR (field_editor));

	/* allocate private structure */
	field_editor->priv = g_new0 (GnomeDbFieldEditorPrivate, 1);
	field_editor->priv->model = NULL;
	field_editor->priv->col = 0;
	field_editor->priv->row = 0;
	field_editor->priv->widget = NULL;
	field_editor->priv->show_label = TRUE;
	field_editor->priv->label_position = GNOME_DB_FIELD_EDITOR_LABEL_ON_TOP;
}

static void
gnome_db_field_editor_set_property (GObject *object,
			    guint param_id,
			    const GValue *value,
			    GParamSpec *pspec)
{
	GnomeDbFieldEditor *field_editor = (GnomeDbFieldEditor *) object;

	g_return_if_fail (GNOME_DB_IS_FIELD_EDITOR (field_editor));

	switch (param_id) {
	default :
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
		break;
	}
}

static void
gnome_db_field_editor_get_property (GObject *object,
			    guint param_id,
			    GValue *value,
			    GParamSpec *pspec)
{
	GnomeDbFieldEditor *field_editor = (GnomeDbFieldEditor *) object;

	g_return_if_fail (GNOME_DB_IS_FIELD_EDITOR (field_editor));

	switch (param_id) {
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
		break;
	}
}

static void
gnome_db_field_editor_finalize (GObject *object)
{
	GnomeDbFieldEditor *field_editor = (GnomeDbFieldEditor *) object;

	g_return_if_fail (GNOME_DB_IS_FIELD_EDITOR (field_editor));

	/* free memory */
	if (GDA_IS_DATA_MODEL (field_editor->priv->model)) {
		g_object_unref (G_OBJECT (field_editor->priv->model));
		field_editor->priv->model = NULL;
	}
	
	if (GTK_IS_WIDGET(field_editor->priv->label)) {
		g_object_unref (G_OBJECT (field_editor->priv->label));
		field_editor->priv->label = NULL;
	}
	
	if (GTK_IS_WIDGET(field_editor->priv->widget)) {
		g_object_unref (G_OBJECT (field_editor->priv->widget));
		field_editor->priv->widget = NULL;
	}
	
	/* chain to parent class */
	parent_class->finalize (object);
}

GType
gnome_db_field_editor_get_type (void)
{
	static GType type = 0;

	if (!type) {
		static const GTypeInfo info = {
			sizeof (GnomeDbFieldEditorClass),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			(GClassInitFunc) gnome_db_field_editor_class_init,
			NULL,
			NULL,
			sizeof (GnomeDbFieldEditor),
			0,
			(GInstanceInitFunc) gnome_db_field_editor_init
		};
		type = g_type_register_static (PARENT_TYPE, "GnomeDbFieldEditor", &info, 0);
	}
	return type;
}

GdaValueType
gnome_db_field_editor_get_value_vtype (GnomeDbFieldEditor *field_editor)
{
	return gda_value_get_vtype(gnome_db_field_editor_get_value(field_editor));
}

/**
 * gnome_db_field_editor_new
 *
 * Create a new #GnomeDbFieldEditor widget, which is a high-level widget which
 * lets you display display and edit a #GdaValue.
 *
 * Returns: the newly created widget.
 */
GtkWidget *
gnome_db_field_editor_new (void)
{
	GnomeDbFieldEditor *field_editor;

	field_editor = g_object_new (GNOME_DB_TYPE_FIELD_EDITOR, NULL);
	return GTK_WIDGET (field_editor);
}

/**
 * gnome_db_field_editor_new_with_model
 *
 * Create a new #GnomeDbFieldEditor widget, which is a high-level widget which
 * lets you display and edit a #GdaValue in a #GdaDataModel.
 * @model: a GdaDataModel to take data form.
 * @col: The column on the GdaDataModel.
 * @row: The current row on the GdaDataModel.
 * Returns: the newly created widget.
 */
GtkWidget *
gnome_db_field_editor_new_with_model (GdaDataModel *model, gint col, gint row)
{
	GnomeDbFieldEditor *field_editor;

	field_editor = g_object_new (GNOME_DB_TYPE_FIELD_EDITOR, NULL);
	gnome_db_field_editor_set_model(field_editor,model, col, row);
	return GTK_WIDGET (field_editor);
}

/**
 * gnome_db_field_editor_get_model
 */
GdaDataModel*
gnome_db_field_editor_get_model (GnomeDbFieldEditor *field_editor)
{
	
	g_return_if_fail (GNOME_DB_IS_FIELD_EDITOR (field_editor));

	return field_editor->priv->model;
}

/**
 * gnome_db_field_editor_get_field_editor_type
 */
void
gnome_db_field_editor_set_model (GnomeDbFieldEditor *field_editor, GdaDataModel *model, gint col, gint row)
{
	
	g_return_if_fail (GNOME_DB_IS_FIELD_EDITOR (field_editor));
	g_return_if_fail (GDA_IS_DATA_MODEL (model));
	g_return_if_fail (col >= 0 && col <= gda_data_model_get_n_columns(model));
	g_return_if_fail (row >= 0 && row <= gda_data_model_get_n_rows(model));
	
	if(GDA_IS_DATA_MODEL(field_editor->priv->model)) 
		g_object_unref(field_editor->priv->model);
		
	field_editor->priv->model = model;
	g_object_ref(model);
	field_editor->priv->col = col;
	field_editor->priv->row = row;
	g_message("Antes de llamara CREATE_WIDGETS\n");
	create_widgets(field_editor);
	g_message("Antes de llamara UPDATE_VALUE\n");
	update_value(field_editor);
	g_signal_emit (G_OBJECT (field_editor), field_editor_signals[MODEL_CHANGED], 0);
}

/**
 * gnome_db_field_editor_get_field_editor_type
 */
GdaValueType
gnome_db_field_editor_get_field_editor_type (GnomeDbFieldEditor *field_editor)
{
	g_return_val_if_fail (GNOME_DB_IS_FIELD_EDITOR (field_editor), -1);
	g_return_val_if_fail (GDA_IS_DATA_MODEL (field_editor->priv->model), -1);
	
	GdaValue *val;
	val	= gnome_db_field_editor_get_value(field_editor);
	
	g_return_val_if_fail (val != NULL, GDA_VALUE_TYPE_UNKNOWN);
	
	g_message("En GetType: Campo: %s   Valor: %s Tipo: %i\n", 
						gnome_db_field_editor_get_column_name(field_editor),
						gda_value_stringify(val),
						gda_value_get_type(val));
	
	return gda_value_get_type(val);
}

/**
 * gnome_db_field_editor_get_value
 * @field_editor: a #GnomeDbFieldEditor widget.
 *
 * Return the #GdaValue being used by the given widget.
 */
GdaValue *
gnome_db_field_editor_get_value (GnomeDbFieldEditor *field_editor)
{
	g_return_val_if_fail (GNOME_DB_IS_FIELD_EDITOR (field_editor), NULL);
	g_return_val_if_fail (GDA_IS_DATA_MODEL(field_editor->priv->model), NULL);
	g_return_val_if_fail (field_editor->priv->col >= 0 && field_editor->priv->col <= gda_data_model_get_n_columns(field_editor->priv->model), -1);
	g_return_val_if_fail (field_editor->priv->row >= 0 && field_editor->priv->row <= gda_data_model_get_n_rows(field_editor->priv->model), -1);
	
	GdaValue *value;
	value = gda_data_model_get_value_at(field_editor->priv->model, 
													field_editor->priv->col, 
													field_editor->priv->row);
	
	return value;
}

/**
 * gnome_db_field_editor_set_value
 * 
 * @field_editor: a #GnomeDbFieldEditor widget.
 *
 * 
 */
void
gnome_db_field_editor_set_value (GnomeDbFieldEditor *field_editor, GdaValue *value)
{
	g_return_if_fail (GNOME_DB_IS_FIELD_EDITOR (field_editor));
	g_return_if_fail (value!= NULL);
	set_value(field_editor, value);
}


/**
 * gnome_db_field_editor_get_text
 * @field_editor: a #GnomeDbFieldEditor widget.
 *
 * Return the string in the #GdaValue.
 */
const gchar *
gnome_db_field_editor_get_value_text (GnomeDbFieldEditor *field_editor)
{
	g_return_val_if_fail (GNOME_DB_IS_FIELD_EDITOR (field_editor), NULL);
	
	GdaValue *val;
	val	= gda_data_model_get_value_at(field_editor->priv->model,
												field_editor->priv->col,
												field_editor->priv->row);
	
	gchar* txt = gda_value_stringify(val);
	

	
	return txt;
}

/**
 * gnome_db_field_editor_get_widget_text
 * @field_editor: a #GnomeDbFieldEditor widget.
 *
 * Return a gchar* showed by the widget.
 */
const gchar *
gnome_db_field_editor_get_widget_text (GnomeDbFieldEditor *field_editor)
{
	g_return_val_if_fail (GNOME_DB_IS_FIELD_EDITOR (field_editor), NULL);
	if(GTK_IS_ENTRY(field_editor->priv->widget))
		return gtk_entry_get_text(GTK_ENTRY(field_editor->priv->widget));
	else
		return NULL;
}

/**
 * gnome_db_field_editor_get_widget_value
 * @field_editor: a #GnomeDbFieldEditor widget.
 *
 * Return a GdaValue* from the text showed by the widget.
 */
GdaValue*
gnome_db_field_editor_get_widget_value (GnomeDbFieldEditor *field_editor)
{
	GdaValue *val;
	g_return_val_if_fail (GNOME_DB_IS_FIELD_EDITOR (field_editor), NULL);
	if(GTK_IS_ENTRY(field_editor->priv->widget))
		
		if(gda_value_set_from_string(val, 
									gtk_entry_get_text(GTK_ENTRY(field_editor->priv->widget)),
									gnome_db_field_editor_get_value_vtype(field_editor)))
			return val;
		else
			/* If the Text is invalid for the type of GdaValue in the Field Editor Return the orignal value */
			return gnome_db_field_editor_get_value(field_editor);
	else
		return NULL;
}


/**
 * gnome_db_field_editor_get_current_row
 * @field_editor: a #GnomeDbFieldEditor widget.
 *
 * Get the row currently being shown by the given field_editor widget.
 *
 * Returns: the current row.
 */
gint
gnome_db_field_editor_get_current_row (GnomeDbFieldEditor *field_editor)
{
	g_return_val_if_fail (GNOME_DB_IS_FIELD_EDITOR (field_editor), -1);
	return field_editor->priv->row;
}

/**
 * gnome_db_field_editor_get_column
 * @field_editor: a #GnomeDbFieldEditor widget.
 * Returns: the number of column of the field.
 *
 * Sets the current row on the given field_editor widget.
 */
gint
gnome_db_field_editor_get_column (GnomeDbFieldEditor *field_editor)
{
	g_return_if_fail (GNOME_DB_IS_FIELD_EDITOR (field_editor));
	return	field_editor->priv->col;
}

/**
 * gnome_db_field_editor_set_column
 * @field_editor: a #GnomeDbFieldEditor widget.
 * @row: the new row to display.
 *
 * Sets the current row on the given field_editor widget.
 */
void
gnome_db_field_editor_set_column (GnomeDbFieldEditor *field_editor, gint col)
{
	g_return_if_fail (GNOME_DB_IS_FIELD_EDITOR (field_editor));
	g_return_if_fail (col >= 0 && gda_data_model_get_n_columns(field_editor->priv->model));
	
	field_editor->priv->col = col;
	update_value(field_editor);
}

void
gnome_db_field_editor_show_label(GnomeDbFieldEditor *field_editor, gboolean show)
{
	g_return_if_fail (GNOME_DB_IS_FIELD_EDITOR (field_editor));
	field_editor->priv->show_label = show;
	create_widgets(field_editor);
}

gboolean
gnome_db_field_editor_is_label_shown(GnomeDbFieldEditor *field_editor)
{
	g_return_if_fail (GNOME_DB_IS_FIELD_EDITOR (field_editor));
	return field_editor->priv->show_label;
}

const char*
gnome_db_field_editor_get_column_name (GnomeDbFieldEditor *field_editor)
{
	return gda_data_model_get_column_title(field_editor->priv->model, field_editor->priv->col);
}

gint
gnome_db_field_editor_get_row (GnomeDbFieldEditor *field_editor)
{
	return field_editor->priv->row;
}
void
gnome_db_field_editor_set_row (GnomeDbFieldEditor *field_editor, gint row)
{
	g_return_if_fail (GNOME_DB_IS_FIELD_EDITOR (field_editor));
	g_return_if_fail (row >= 0 && row <= gda_data_model_get_n_rows(field_editor->priv->model));
	
	field_editor->priv->row = row;
	update_value(field_editor);
}

GnomeDbFieldEditorLabelPosition
gnome_db_field_editor_get_label_position(GnomeDbFieldEditor *field_editor)
{
	g_return_if_fail (GNOME_DB_IS_FIELD_EDITOR (field_editor));
	return field_editor->priv->label_position;
}
void
gnome_db_field_editor_set_label_position(GnomeDbFieldEditor *field_editor, GnomeDbFieldEditorLabelPosition pos)
{
	g_return_if_fail (GNOME_DB_IS_FIELD_EDITOR (field_editor));
	field_editor->priv->label_position = pos;
	create_widgets(field_editor);
}
