/* GNOME DB library
 * Copyright (C) 1999-2002 The GNOME Foundation.
 *
 * AUTHORS:
 *      Daniel Espinosa <esodan@yahoo.com.mx>
 *
 * This Library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This Library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this Library; see the file COPYING.LIB.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#if !defined(__gnome_db_field_editor_h__)
#  define __gnome_db_field_editor_h__

#include <libgda/libgda.h>
#include <gtk/gtk.h>

G_BEGIN_DECLS

#define GNOME_DB_TYPE_FIELD_EDITOR            (gnome_db_form_get_type())
#define GNOME_DB_FIELD_EDITOR(obj)            (G_TYPE_CHECK_INSTANCE_CAST (obj, GNOME_DB_TYPE_FIELD_EDITOR, GnomeDbFieldEditor))
#define GNOME_DB_FIELD_EDITOR_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST (klass, GNOME_DB_TYPE_FIELD_EDITOR, GnomeDbFieldEditorClass))
#define GNOME_DB_IS_FIELD_EDITOR(obj)         (G_TYPE_CHECK_INSTANCE_TYPE (obj, GNOME_DB_TYPE_FIELD_EDITOR))
#define GNOME_DB_IS_FIELD_EDITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GNOME_DB_TYPE_FIELD_EDITOR))

typedef struct _GnomeDbFieldEditor        GnomeDbFieldEditor;
typedef struct _GnomeDbFieldEditorClass   GnomeDbFieldEditorClass;
typedef struct _GnomeDbFieldEditorPrivate GnomeDbFieldEditorPrivate;

struct _GnomeDbFieldEditor {
	GtkVBox frame;
	GnomeDbFieldEditorPrivate *priv;
};

struct _GnomeDbFieldEditorClass {
	GtkVBoxClass parent_class;

	/* signals */
	void (* model_changed) (GnomeDbFieldEditor *field_editor);
};


typedef enum {
	GNOME_DB_FIELD_EDITOR_LABEL_ON_TOP,
	GNOME_DB_FIELD_EDITOR_LABEL_BEHIND,
	GNOME_DB_FIELD_EDITOR_LABEL_LEFT,
	GNOME_DB_FIELD_EDITOR_LABEL_RIGHT
} GnomeDbFieldEditorLabelPosition;

GType            gnome_db_field_editor_get_type (void);
GdaValueType     gnome_db_field_editor_get_value_vtype (GnomeDbFieldEditor *field_editor);
GtkWidget*       gnome_db_field_editor_new (void);
GtkWidget*       gnome_db_field_editor_new_with_model (GdaDataModel *model, gint col, gint row);
GdaDataModel*    gnome_db_field_editor_get_model(GnomeDbFieldEditor *field_editor);
void			 gnome_db_field_editor_set_model(GnomeDbFieldEditor *field_editor, GdaDataModel *model, gint col, gint row);
GdaValueType     gnome_db_field_editor_get_field_editor_type (GnomeDbFieldEditor *field_editor);
GdaValue*	     gnome_db_field_editor_get_value (GnomeDbFieldEditor *field_editor);
void		     gnome_db_field_editor_set_value (GnomeDbFieldEditor *field_editor, GdaValue *value);
const gchar*     gnome_db_field_editor_get_value_text (GnomeDbFieldEditor *field_editor);
const gchar*     gnome_db_field_editor_get_widget_text (GnomeDbFieldEditor *field_editor);
gint             gnome_db_field_editor_get_row (GnomeDbFieldEditor *field_editor);
void             gnome_db_field_editor_set_row (GnomeDbFieldEditor *field_editor, gint row);
gint             gnome_db_field_editor_get_column (GnomeDbFieldEditor *field_editor);
void             gnome_db_field_editor_set_column (GnomeDbFieldEditor *field_editor, gint col);
const char*		 gnome_db_field_editor_get_column_name (GnomeDbFieldEditor *field_editor);
void             gnome_db_field_editor_show_label(GnomeDbFieldEditor *field_editor, gboolean show);
gboolean         gnome_db_field_editor_is_label_shown(GnomeDbFieldEditor *field_editor);
GnomeDbFieldEditorLabelPosition gnome_db_field_editor_get_label_position(GnomeDbFieldEditor *field_editor);
void             gnome_db_field_editor_set_label_position(GnomeDbFieldEditor *field_editor, GnomeDbFieldEditorLabelPosition pos);

G_END_DECLS

#endif
