/* GNOME DB library
 * Copyright (C) 1999-2002 The GNOME Foundation.
 *
 * AUTHORS:
 *      Rodrigo Moya <rodrigo@gnome-db.org>
 *		Daniel Espinosa <esodan@yahoo.com.mx>
 *
 * This Library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This Library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this Library; see the file COPYING.LIB.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#ifndef GNOME_DB_FORM_EDITOR_H
#define GNOME_DB_FORM_EDITOR_H

#include <libgda/libgda.h>
#include <gtk/gtk.h>

G_BEGIN_DECLS

#define GNOME_DB_TYPE_FORM_EDITOR            (gnome_db_form_editor_get_type())
#define GNOME_DB_FORM_EDITOR(obj)            (G_TYPE_CHECK_INSTANCE_CAST (obj, GNOME_DB_TYPE_FORM_EDITOR, GnomeDbFormEditor))
#define GNOME_DB_FORM_EDITOR_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST (klass, GNOME_DB_TYPE_FORM_EDITOR, GnomeDbFormEditorClass))
#define GNOME_DB_IS_FORM_EDITOR(obj)         (G_TYPE_CHECK_INSTANCE_TYPE (obj, GNOME_DB_TYPE_FORM_EDITOR))
#define GNOME_DB_IS_FORM_EDITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GNOME_DB_TYPE_FORM_EDITOR))

typedef struct _GnomeDbFormEditor  GnomeDbFormEditor;
typedef struct _GnomeDbFormEditorClass   GnomeDbFormEditorClass;
typedef struct _GnomeDbFormEditorPrivate GnomeDbFormEditorPrivate;

struct _GnomeDbFormEditor {
	GtkVBox frame;
	GnomeDbFormEditorPrivate *priv;
};

struct _GnomeDbFormEditorClass {
	GtkVBoxClass parent_class;

	/* signals */
	void (* model_changed) (GnomeDbFormEditor *form);
	void (* row_changed) (GnomeDbFormEditor *form);
};

typedef enum {
	GNOME_DB_FORM_EDITOR_TYPE_NAVIGATOR,
	GNOME_DB_FORM_EDITOR_TYPE_LIST
} GnomeDbFormEditorType;



GType            gnome_db_form_editor_get_type (void);
GtkWidget       *gnome_db_form_editor_new (void);
GtkWidget       *gnome_db_form_editor_new_with_model(GdaDataModel *model, gint row, gboolean columned);
GnomeDbFormEditorType  gnome_db_form_editor_get_form_type (GnomeDbFormEditor *form);
GtkWidget*       gnome_db_form_editor_find_field (GnomeDbFormEditor *form, const gchar *name);
void             gnome_db_form_editor_update_data(GnomeDbFormEditor *form);
void             gnome_db_form_editor_delete_current_row (GnomeDbFormEditor *form);
void             gnome_db_form_editor_set_form_type (GnomeDbFormEditor *form, GnomeDbFormEditorType type);
GdaDataModel    *gnome_db_form_editor_get_model (GnomeDbFormEditor *form);
void             gnome_db_form_editor_set_model (GnomeDbFormEditor *form, GdaDataModel *model);
void             gnome_db_form_editor_get_current_row(GnomeDbFormEditor *form);
void             gnome_db_form_editor_set_current_row(GnomeDbFormEditor *form, gint row);
void             gnome_db_form_editor_free(GnomeDbFormEditor *form);

G_END_DECLS

#endif
